@@ -1,3 +1,4 @@
+from __future__ import print_function, division, absolute_import
 
 import atexit
 import json
@@ -9,17 +10,20 @@ import sys
 
 import bokeh
 import distributed.bokeh
-from toolz import get_in
+from toolz import get_in, concat
 
 from ..utils import ignoring
 from ..compatibility import logging_names
 from ..config import config
 
+
 dirname = os.path.dirname(distributed.__file__)
 paths = [os.path.join(dirname, 'bokeh', name)
-         for name in ['status', 'tasks', 'workers']]
+         for name in ['background', 'status', 'tasks', 'workers',
+             'memory-usage.py', 'task-stream.py', 'task-progress.py',
+             'resource-profiles.py', 'worker-table.py', 'processing-stacks.py']]
 
-logger = logging.getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 dask_dir = os.path.join(os.path.expanduser('~'), '.dask')
 if not os.path.exists(dask_dir):
@@ -74,8 +78,8 @@ class BokehWebInterface(object):
                          'http-port': http_port,
                          'tcp-port': tcp_port,
                          'bokeh-port': bokeh_port}
-        with open(os.path.join(dask_dir, '.dask-web-ui.json'), 'w') as f:
-            json.dump(bokeh_options, f, indent=2)
+
+        args.extend(['--args'] + list(map(str, concat(bokeh_options.items()))))
 
         import subprocess
         process = subprocess.Popen(args)
@@ -83,12 +87,12 @@ class BokehWebInterface(object):
         def cleanup_process():
             try:
                 process.terminate()
-            except OSError():
+            except OSError:
                 pass
         atexit.register(cleanup_process)
 
         if not quiet:
-            logger.info(" Bokeh UI at:  http://%s:%d/status/"
+            logger.info("Web UI: http://%s:%d/status/"
                          % (ip, bokeh_port))
 
     def close(self, join=True, timeout=None):
