@@ -18,6 +18,8 @@ from bokeh.plotting import figure
 from distributed.diagnostics.progress_stream import progress_quads, nbytes_bar
 from distributed.utils import log_errors
 
+# from .export_tool import ExportTool
+
 
 class DashboardComponent(object):
     """ Base class for Dask.distributed UI dashboard components.
@@ -44,11 +46,12 @@ class TaskStream(DashboardComponent):
 
     The start and stop time of tasks as they occur on each core of the cluster.
     """
-    def __init__(self, n_rectangles=1000, **kwargs):
+    def __init__(self, n_rectangles=1000, clear_interval=20000, **kwargs):
         """
         kwargs are applied to the bokeh.models.plots.Plot constructor
         """
         self.n_rectangles = n_rectangles
+        self.clear_interval = clear_interval
 
         self.source = ColumnDataSource(data=dict(
             start=[], duration=[], key=[], name=[], color=[],
@@ -87,8 +90,15 @@ class TaskStream(DashboardComponent):
                 """
         )
 
+        # export = ExportTool()
+        # export.register_plot(self.root)
+
         self.root.add_tools(
-            hover, ResetTool(), PanTool(dimensions="width"), WheelZoomTool(dimensions="width")
+            hover,
+            # export,
+            ResetTool(reset_size=False),
+            PanTool(dimensions="width"),
+            WheelZoomTool(dimensions="width")
         )
 
         # Required for update callback
@@ -107,10 +117,10 @@ class TaskStream(DashboardComponent):
                           for k, v in rectangles.items()}
             self.task_stream_index[0] = index[-1]
 
-            # If there has been a five second delay, clear old rectangles
+            # If there has been a significant delay then clear old rectangles
             if rectangles['start']:
                 last_end = old['start'][ind - 1] + old['duration'][ind - 1]
-                if min(rectangles['start']) > last_end + 20000:  # long delay
+                if min(rectangles['start']) > last_end + self.clear_interval:
                     self.source.data.update(rectangles)
                     return
 
@@ -277,7 +287,7 @@ class ResourceProfiles(DashboardComponent):
         )
         g2 = resource_plot.add_glyph(
             self.source,
-            Line(x='time', y='cpu', line_color="#1f78b4"    , **line_opts)
+            Line(x='time', y='cpu', line_color="#1f78b4", **line_opts)
         )
 
         resource_plot.add_layout(
@@ -400,7 +410,6 @@ class WorkerTable(DashboardComponent):
             Circle(x='memory_percent', y=0, size=10, fill_alpha=0.5)
         )
 
-        mem_plot.add_layout(LinearAxis(), 'left')
         mem_plot.add_layout(LinearAxis(), 'below')
 
         hover = HoverTool(
