@@ -51,6 +51,12 @@ def main(host, port, http_port, bokeh_port, show, _bokeh,
                 os.remove(pid_file)
         atexit.register(del_pid_file)
 
+    if sys.platform.startswith('linux'):
+        import resource   # module fails importing on Windows
+        soft, hard = resource.getrlimit(resource.RLIMIT_NOFILE)
+        limit = max(soft, hard // 2)
+        resource.setrlimit(resource.RLIMIT_NOFILE, (limit, hard))
+
     given_host = host
     host = host or get_ip()
     if ':' in host and port == 8786:
@@ -58,6 +64,7 @@ def main(host, port, http_port, bokeh_port, show, _bokeh,
         port = int(port)
     ip = socket.gethostbyname(host)
     loop = IOLoop.current()
+    logger.info('-' * 47)
     scheduler = Scheduler(ip=ip, loop=loop,
                           services={('http', http_port): HTTPScheduler})
     scheduler.start(port)
@@ -75,13 +82,17 @@ def main(host, port, http_port, bokeh_port, show, _bokeh,
         except Exception as e:
             logger.warn("Could not start Bokeh web UI", exc_info=True)
 
-    loop.start()
-    loop.close()
-    scheduler.stop()
-    if bokeh_proc:
-        bokeh_proc.close()
 
-    logger.info("End scheduler at %s:%d", ip, port)
+    logger.info('-' * 47)
+    try:
+        loop.start()
+        loop.close()
+    finally:
+        scheduler.stop()
+        if bokeh_proc:
+            bokeh_proc.close()
+
+        logger.info("End scheduler at %s:%d", ip, port)
 
 
 def go():
