@@ -10,7 +10,6 @@ from glob import glob
 import logging
 import os
 import sys
-import pickle
 from time import sleep
 import uuid
 from threading import Thread
@@ -19,7 +18,7 @@ import socket
 
 import dask
 from dask.base import tokenize, normalize_token, Base
-from dask.core import flatten, _deps
+from dask.core import flatten, get_dependencies
 from dask.compatibility import apply
 from dask.context import _globals
 from toolz import first, groupby, merge, valmap, keymap
@@ -33,11 +32,13 @@ from tornado.queues import Queue
 from .batched import BatchedSend
 from .utils_comm import WrappedKey, unpack_remotedata, pack_data
 from .compatibility import Queue as pyQueue, Empty, isqueue
-from .core import (read, write, connect, coerce_to_rpc, dumps,
-        clean_exception, loads)
+from .core import (read, write, connect, coerce_to_rpc, clean_exception)
+from .protocol import to_serialize
+from .protocol.pickle import dumps, loads
 from .worker import dumps_function, dumps_task
 from .utils import (All, sync, funcname, ignoring, queue_to_iterator,
         tokey, log_errors, str_graph)
+from .versions import get_versions
 
 logger = logging.getLogger(__name__)
 
@@ -151,6 +152,15 @@ class Future(WrappedKey):
         """
         return sync(self.client.loop, self._exception)
 
+    def add_done_callback(self, fn):
+        """ Call callback on future when callback has finished
+
+        The callback ``fn`` should take the future as its only argument.  This
+        will be called regardless of if the future completes successfully,
+        errs, or is cancelled
+        """
+        self.client.loop.add_callback(done_callback, self, fn)
+
     def cancel(self):
         """ Returns True if the future has been cancelled """
         return self.client.cancel([self])
@@ -225,6 +235,14 @@ class Future(WrappedKey):
     __repr__ = __str__
 
 
+@gen.coroutine
+def done_callback(future, callback):
+    """ Wait on future, then call callback """
+    while future.status == 'pending':
+        yield future.event.wait()
+    callback(future)
+
+
 @partial(normalize_token.register, Future)
 def normalize_future(f):
     return [f.key, type(f)]
@@ -340,6 +358,9 @@ class Client(object):
                 self.cluster = LocalCluster(scheduler_port=0, loop=self.loop,
                                             start=False)
             self._start_arg = self.cluster.scheduler_address
+            while (not self.cluster.workers or
+               len(self.cluster.scheduler.ncores) < len(self.cluster.workers)):
+                yield gen.sleep(0.01)
 
         self.scheduler = coerce_to_rpc(self._start_arg, timeout=timeout)
         self.scheduler_stream = None
@@ -518,16 +539,24 @@ class Client(object):
                 self.scheduler.close_rpc()
 
     def shutdown(self, timeout=10):
-        """ Send shutdown signal and wait until scheduler terminates """
+        """ Send shutdown signal and wait until scheduler terminates
+
+        This cancels all currently running tasks, clears the state of the
+        scheduler, and shuts down all workers and scheduler.
+
+        You do not need to call this when you finish your session.  You only
+        need to call this if you want to take down the distributed cluster.
+
+        See Also
+        --------
+        Client.restart
+        """
+        # XXX handling of self.status here is not thread-safe
         if self.status == 'closed':
             return
-        self.status = 'closed'
-        with ignoring(AttributeError):
-            self.loop.add_callback(self.scheduler_stream.send,
-                                   {'op': 'close-stream'})
-            sync(self.loop, self.scheduler_stream.close)
-        with ignoring(AttributeError):
-            self.scheduler.close_rpc()
+        sync(self.loop, self._shutdown, fast=True)
+        assert self.status == 'closed'
+
         if self._should_close_loop:
             sync(self.loop, self.loop.stop)
             self.loop.close(all_fds=True)
@@ -536,8 +565,6 @@ class Client(object):
             dask.set_options(get=self._previous_get)
         with ignoring(AttributeError):
             dask.set_options(shuffle=self._previous_shuffle)
-        if _global_client[0] is self:
-            _global_client[0] = None
         if self.get == _globals.get('get'):
             del _globals['get']
         with ignoring(AttributeError):
@@ -647,6 +674,8 @@ class Client(object):
         ----------
         func: callable
         iterables: Iterables, Iterators, or Queues
+        key: str, list
+            Prefix for task names if string.  Explicit names if list.
         pure: bool (defaults to True)
             Whether or not the function is pure.  Set ``pure=False`` for
             impure functions like ``np.random.random``.
@@ -683,6 +712,8 @@ class Client(object):
             else:
                 return queue_to_iterator(q_out)
 
+        key = kwargs.pop('key', None)
+        key = key or funcname(func)
         pure = kwargs.pop('pure', True)
         workers = kwargs.pop('workers', None)
         allow_other_workers = kwargs.pop('allow_other_workers', False)
@@ -691,13 +722,16 @@ class Client(object):
             raise ValueError("Only use allow_other_workers= if using workers=")
 
         iterables = list(zip(*zip(*iterables)))
-        if pure:
-            keys = [funcname(func) + '-' + tokenize(func, kwargs, *args)
-                    for args in zip(*iterables)]
+        if isinstance(key, list):
+            keys = key
         else:
-            uid = str(uuid.uuid4())
-            keys = [funcname(func) + '-' + uid + '-' + str(i)
-                    for i in range(min(map(len, iterables)))]
+            if pure:
+                keys = [key + '-' + tokenize(func, kwargs, *args)
+                        for args in zip(*iterables)]
+            else:
+                uid = str(uuid.uuid4())
+                keys = [key + '-' + uid + '-' + str(i)
+                        for i in range(min(map(len, iterables)))] if iterables else []
 
         if not kwargs:
             dsk = {key: (func,) + args
@@ -791,7 +825,7 @@ class Client(object):
         if bad_data and errors == 'skip' and isinstance(futures2, list):
             futures2 = [f for f in futures2 if f not in bad_data]
 
-        data = valmap(loads, response['data'])
+        data = response['data']
         result = pack_data(futures2, merge(data, bad_data))
         raise gen.Return(result)
 
@@ -858,15 +892,19 @@ class Client(object):
             raise gen.Return({k: d[tokey(k)] for k in data})
 
         if isinstance(data, dict):
-            data2 = valmap(dumps, data)
+            data2 = valmap(to_serialize, data)
+            types = valmap(type, data)
         elif isinstance(data, (list, tuple, set, frozenset)):
-            data2 = list(map(dumps, data))
+            data2 = list(map(to_serialize, data))
+            types = list(map(type, data))
         elif isinstance(data, (Iterable, Iterator)):
-            data2 = list(map(dumps, data))
+            data2 = list(map(to_serialize, data))
+            types = list(map(type, data))
         else:
             raise TypeError("Don't know how to scatter %s" % type(data))
         keys = yield self.scheduler.scatter(data=data2, workers=workers,
-                                            client=self.id, broadcast=broadcast)
+                                            client=self.id,
+                                            broadcast=broadcast)
         if isinstance(data, dict):
             out = {k: Future(k, self) for k in keys}
         elif isinstance(data, (tuple, list, set, frozenset)):
@@ -881,6 +919,13 @@ class Client(object):
             self.futures[key]['status'] = 'finished'
             self.futures[key]['event'].set()
 
+        if isinstance(types, list):
+            for key, typ in zip(keys, types):
+                self.futures[key]['type'] = typ
+        elif isinstance(types, dict):
+            for key in keys:
+                self.futures[key]['type'] = types[key]
+
         raise gen.Return(out)
 
     def _threaded_scatter(self, q_or_i, qout, **kwargs):
@@ -907,6 +952,11 @@ class Client(object):
     def scatter(self, data, workers=None, broadcast=False, maxsize=0):
         """ Scatter data into distributed memory
 
+        This moves data from the local client process into the workers of the
+        distributed scheduler.  Note that it is often better to submit jobs to
+        your workers to have them load the data rather than loading data
+        locally and then scattering it out to them.
+
         Parameters
         ----------
         data: list, iterator, dict, or Queue
@@ -938,14 +988,17 @@ class Client(object):
          'z': <Future: status: finished, key: z>}
 
         Constrain location of data to subset of workers
+
         >>> c.scatter([1, 2, 3], workers=[('hostname', 8788)])   # doctest: +SKIP
 
         Handle streaming sequences of data with iterators or queues
+
         >>> seq = c.scatter(iter([1, 2, 3]))  # doctest: +SKIP
         >>> next(seq)  # doctest: +SKIP
         <Future: status: finished, key: c0a8a20f903a4915b94db8de3ea63195>,
 
         Broadcast data to all workers
+
         >>> [future] = c.scatter([element], broadcast=True)  # doctest: +SKIP
 
         See Also
@@ -1020,11 +1073,13 @@ class Client(object):
         Examples
         --------
         Publishing client:
+
         >>> df = dd.read_csv('s3://...')  # doctest: +SKIP
         >>> df = c.persist(df) # doctest: +SKIP
         >>> c.publish_dataset(my_dataset=df)  # doctest: +SKIP
 
         Receiving client:
+
         >>> c.list_datasets()  # doctest: +SKIP
         ['my_dataset']
         >>> df2 = c.get_dataset('my_dataset')  # doctest: +SKIP
@@ -1102,7 +1157,7 @@ class Client(object):
         results = {}
         for key, resp in responses.items():
             if resp['status'] == 'OK':
-                results[key] = loads(resp['result'])
+                results[key] = resp['result']
             elif resp['status'] == 'error':
                 raise loads(resp['exception'])
         raise Return(results)
@@ -1174,7 +1229,7 @@ class Client(object):
                     raise CancelledError(v)
 
         for k, v in dsk3.items():
-            dependencies[k] |= set(_deps(dsk3, v))
+            dependencies[k] |= get_dependencies(dsk3, task=v)
 
         self._send_to_scheduler({'op': 'update-graph',
                                  'tasks': valmap(dumps_task, dsk3),
@@ -1243,6 +1298,59 @@ class Client(object):
         results2 = pack_data(keys, results)
         return results2
 
+    def _optimize_insert_futures(self, dsk, keys):
+        """ Replace known keys in dask graph with Futures
+
+        When given a Dask graph that might have overlapping keys with our known
+        results we replace the values of that graph with futures.  This can be
+        used as an optimization to avoid recomputation.
+
+        This returns the same graph if unchanged but a new graph if any changes
+        were necessary.
+        """
+        changed = False
+        for key in list(dsk):
+            if tokey(key) in self.futures:
+                if not changed:
+                    changed = True
+                    dsk = dsk.copy()
+                dsk[key] = Future(key, self)
+
+        if changed:
+            dsk, _ = dask.optimize.cull(dsk, keys)
+
+        return dsk
+
+    def normalize_collection(self, collection):
+        """
+        Replace collection's tasks by already existing futures if they exist
+
+        This normalizes the tasks within a collections task graph against the
+        known futures within the scheduler.  It returns a copy of the
+        collection with a task graph that includes the overlapping futures.
+
+        Examples
+        --------
+        >>> len(x.dask)  # x is a dask collection with 100 tasks
+        100
+        >>> set(client.futures).intersection(x.dask)  # some overlap exists
+        10
+
+        >>> x = client.normalize_collection(x)
+        >>> len(x.dask)  # smaller computational graph
+        20
+
+        See Also
+        --------
+        Client.persist: trigger computation of collection's tasks
+        """
+        dsk = self._optimize_insert_futures(collection.dask, collection._keys())
+
+        if dsk is collection.dask:
+            return collection
+        else:
+            return redict_collection(collection, dsk)
+
     def compute(self, collections, sync=False, optimize_graph=True,
             workers=None, allow_other_workers=False, **kwargs):
         """ Compute dask collections on cluster
@@ -1302,18 +1410,10 @@ class Client(object):
 
         variables = [a for a in collections if isinstance(a, Base)]
 
-        if optimize_graph:
-            groups = groupby(lambda x: x._optimize, variables)
-            dsk = merge([opt(merge([v.dask for v in val]),
-                             [v._keys() for v in val], **kwargs)
-                        for opt, val in groups.items()])
-        else:
-            dsk = merge(c.dask for c in variables)
-
+        dsk = collections_to_dsk(variables, optimize_graph, **kwargs)
         names = ['finalize-%s' % tokenize(v) for v in variables]
         dsk2 = {name: (v._finalize, v._keys()) for name, v in zip(names, variables)}
 
-
         restrictions, loose_restrictions = get_restrictions(collections,
                 workers, allow_other_workers)
 
@@ -1387,13 +1487,7 @@ class Client(object):
 
         assert all(isinstance(c, Base) for c in collections)
 
-        if optimize_graph:
-            groups = groupby(lambda x: x._optimize, collections)
-            dsk = merge([opt(merge([v.dask for v in val]),
-                             [v._keys() for v in val], **kwargs)
-                        for opt, val in groups.items()])
-        else:
-            dsk = merge(c.dask for c in collections)
+        dsk = collections_to_dsk(collections, optimize_graph, **kwargs)
 
         names = {k for c in collections for k in flatten(c._keys())}
 
@@ -1469,7 +1563,7 @@ class Client(object):
         _, fn = os.path.split(filename)
         d = yield self.scheduler.broadcast(msg={'op': 'upload_file',
                                                 'filename': fn,
-                                                'data': data})
+                                                'data': to_serialize(data)})
 
         if any(v['status'] == 'error' for v in d.values()):
             exceptions = [loads(v['exception']) for v in d.values()
@@ -1568,11 +1662,12 @@ class Client(object):
     def replicate(self, futures, n=None, workers=None, branching_factor=2):
         """ Set replication of futures within network
 
-        This performs a tree copy of the data throughout the network
-        individually on each piece of data.
+        Copy data onto many workers.  This helps to broadcast frequently
+        accessed data and it helps to improve resilience.
 
-        This operation blocks until complete.  It does not guarantee
-        replication of data to future workers.
+        This performs a tree copy of the data throughout the network
+        individually on each piece of data.  This operation blocks until
+        complete.  It does not guarantee replication of data to future workers.
 
         Parameters
         ----------
@@ -1804,6 +1899,31 @@ class Client(object):
         """
         return sync(self.loop, self.scheduler.identity)
 
+    def get_versions(self):
+        """ Return version info for the scheduler, all workers and myself
+
+        Examples
+        --------
+        >>> c.get_versions()  # doctest: +SKIP
+        """
+        client = get_versions()
+        try:
+            scheduler = sync(self.loop, self.scheduler.versions)
+        except KeyError:
+            scheduler = None
+
+        def f(worker=None):
+
+            # use our local version
+            try:
+                from distributed.versions import get_versions
+                return get_versions()
+            except ImportError:
+                return None
+
+        workers = sync(self.loop, self._run, f)
+        return {'scheduler': scheduler, 'workers': workers, 'client': client}
+
     def futures_of(self, futures):
         return futures_of(futures, client=self)
 
@@ -1831,7 +1951,8 @@ class Client(object):
 
         magic_names: str or list(str) (optional)
             If defined, register IPython magics with these names for
-            executing code on the workers.
+            executing code on the workers.  If string has asterix then expand
+            asterix into 0, 1, ..., n for n workers
 
         qtconsole: bool (optional)
             If True, launch a Jupyter QtConsole connected to the worker(s).
@@ -1851,6 +1972,14 @@ class Client(object):
 
         >>> c.start_ipython_workers('192.168.1.101:5752', qtconsole=True) # doctest: +SKIP
 
+        Add asterix * in magic names to add one magic per worker
+
+        >>> c.start_ipython_workers(magic_names='w_*') # doctest: +SKIP
+        >>> %w_0 worker.data  # doctest: +SKIP
+        {'x': 1, 'y': 100}
+        >>> %w_1 worker.data  # doctest: +SKIP
+        {'z': 5}
+
         Returns
         -------
         iter_connection_info: list
@@ -1861,14 +1990,18 @@ class Client(object):
         --------
         Client.start_ipython_scheduler: start ipython on the scheduler
         """
-
-        if magic_names and isinstance(magic_names, six.string_types):
-            magic_names = [magic_names]
         if isinstance(workers, six.string_types):
             workers = [workers]
 
         (workers, info_dict) = sync(self.loop, self._start_ipython_workers, workers)
 
+        if magic_names and isinstance(magic_names, six.string_types):
+            if '*' in magic_names:
+                magic_names = [magic_names.replace('*', str(i))
+                                for i in range(len(workers))]
+            else:
+                magic_names = [magic_names]
+
         if 'IPython' in sys.modules:
             from ._ipython_utils import register_remote_magic
             register_remote_magic()
@@ -2084,8 +2217,7 @@ def ensure_default_get(client):
 def redict_collection(c, dsk):
     from dask.delayed import Delayed
     if isinstance(c, Delayed):
-        assert len(dsk) == 1
-        return Delayed(first(dsk), [dsk])
+        return Delayed(c.key, [dsk])
     else:
         cc = copy.copy(c)
         cc.dask = dsk
@@ -2158,3 +2290,24 @@ def get_restrictions(collections, workers, allow_other_workers):
         loose_restrictions = []
 
     return restrictions, loose_restrictions
+
+
+def collections_to_dsk(collections, optimize_graph=True, **kwargs):
+    """
+    Convert many collections into a single dask graph, after optimization
+    """
+    optimizations = _globals.get('optimizations', [])
+    if optimize_graph:
+        groups = groupby(lambda x: x._optimize, collections)
+        groups = {opt: [merge([v.dask for v in val]),
+                       [v._keys() for v in val]]
+                  for opt, val in groups.items()}
+        for opt in optimizations:
+            groups = {k: [opt(dsk, keys), keys]
+                      for k, (dsk, keys) in groups.items()}
+        dsk = merge([opt(dsk, keys, **kwargs)
+                     for opt, (dsk, keys) in groups.items()])
+    else:
+        dsk = merge(c.dask for c in collections)
+
+    return dsk
