@@ -17,12 +17,25 @@ from . import pickle
 from ..utils import log_errors, ensure_bytes
 
 
+def itemsize(dt):
+    """ Itemsize of dtype
+
+    Try to return the itemsize of the base element, return 8 as a fallback
+    """
+    result = dt.base.itemsize
+    if result > 255:
+        result = 8
+    return result
+
+
 def serialize_numpy_ndarray(x):
     if x.dtype.hasobject:
         header = {'pickle': True}
         frames = [pickle.dumps(x)]
         return header, frames
 
+    size = itemsize(x.dtype)
+
     if x.dtype.kind == 'V':
         dt = x.dtype.descr
     else:
@@ -34,15 +47,17 @@ def serialize_numpy_ndarray(x):
               'strides': x.strides,
               'shape': x.shape}
 
+    data = x.view('u1').data
+
     if blosc:
-        frames = frame_split_size([x.data])
+        frames = frame_split_size([data])
         if sys.version_info.major == 2:
             frames = [ensure_bytes(frame) for frame in frames]
-        frames = [blosc.compress(frame, typesize=x.dtype.itemsize,
+        frames = [blosc.compress(frame, typesize=size,
                                  cname='lz4', clevel=5) for frame in frames]
         header['compression'] = ['blosc'] * len(frames)
     else:
-        frames = [x.data]
+        frames = [data]
 
     header['lengths'] = [x.nbytes]
 
