@@ -5,7 +5,7 @@ import random
 from time import time
 
 import pytest
-from toolz import first
+from toolz import first, assoc
 from tornado import gen
 from tornado.tcpserver import TCPServer
 from tornado.tcpclient import TCPClient
@@ -32,9 +32,12 @@ class EchoServer(TCPServer):
     @gen.coroutine
     def handle_stream(self, stream, address):
         while True:
-            msg = yield read(stream)
-            self.count += 1
-            yield write(stream, msg)
+            try:
+                msg = yield read(stream)
+                self.count += 1
+                yield write(stream, msg)
+            except StreamClosedError as e:
+                pass
 
     def listen(self, port=0):
         while True:
@@ -229,7 +232,6 @@ def test_stress():
         def recv():
             while True:
                 result = yield gen.with_timeout(timedelta(seconds=1), read(stream))
-                print(result)
                 L.extend(result)
                 if result[-1] == 9999:
                     break
@@ -238,3 +240,43 @@ def test_stress():
 
         assert L == list(range(0, 10000, 1))
         stream.close()
+
+
+@gen_test()
+def test_sending_traffic_jam():
+    np = pytest.importorskip('numpy')
+    from distributed.protocol import to_serialize
+    data = bytes(np.random.randint(0, 255, size=(300000,)).astype('u1').data)
+    with echo_server() as e:
+        client = TCPClient()
+        stream = yield client.connect('127.0.0.1', e.port)
+
+        b = BatchedSend(interval=0.01)
+        b.start(stream)
+        yield b.last_send
+
+        n = 50
+
+        msg = {'x': to_serialize(data)}
+        for i in range(n):
+            b.send(assoc(msg, 'i', i))
+            print(len(b.buffer))
+            yield gen.sleep(0.001)
+
+        results = []
+        count = 0
+        while len(results) < n:
+            # If this times out then I think it's a backpressure issue
+            # Somehow we're able to flood the socket so that the receiving end
+            # loses some of our messages
+            L = yield gen.with_timeout(timedelta(seconds=5), read(stream))
+            count += 1
+            results.extend(L)
+
+        assert count == b.batch_count == e.count
+        assert b.message_count == n
+
+        assert [r['i'] for r in results] == list(range(50))
+
+        stream.close()  # external closing
+        yield b.close(ignore_closed=True)
